@@ -8,6 +8,7 @@ from bnlearn.bnlearn import (
     to_undirected,
     compare_networks,
     plot,
+    plot_graphviz,
     adjmat2vec,
     adjmat2dict,
     vec2adjmat,
@@ -33,30 +34,40 @@ import bnlearn.parameter_learning as parameter_learning
 import bnlearn.inference as inference
 import bnlearn.network as network
 import bnlearn.confmatrix as confmatrix
+from bnlearn.discretize.discretize import discretize, discretize_value
+from bnlearn.impute import knn_imputer, mice_imputer
 from packaging import version
 
 __author__ = 'Erdogan Tasksen'
 __email__ = 'erdogant@gmail.com'
-__version__ = '0.7.13'
-
-try:
-    import pgmpy
-except:
-    raise ImportError('[bnlearn] >Error: pgmpy version "0.1.13" or higher must be installed manually. Try to: <conda install -c ankurankan pgmpy> or <pip install -U pgmpy>=0.1.13>')
+__version__ = '0.10.0'
 
+import pgmpy
 # Check version pgmpy
-if version.parse(pgmpy.__version__) < version.parse("0.1.13"):
-    raise ImportError('[bnlearn] >Error: This release requires pgmpy to be version >= 0.1.13. Try to: <conda install -c ankurankan pgmpy> or <pip install -U pgmpy>=0.1.13>')
+if version.parse(pgmpy.__version__) < version.parse("0.1.18"):
+    raise ImportError('[bnlearn] >Error: This release requires pgmpy to be version >= 0.1.18. Try to: <pip install -U pgmpy>=0.1.18>')
 
 # Version check
 import matplotlib
 if not version.parse(matplotlib.__version__) >= version.parse("3.3.4"):
-    raise ImportError('[bnlearn] >Error: Matplotlib version should be >= v3.3.4.\nTry to: pip install -U matplotlib')
+    raise ImportError('[bnlearn] >Error: Matplotlib version should be >= v3.3.4\nTry to: pip install -U matplotlib')
 
 import networkx as nx
-if not version.parse(nx.__version__) >= version.parse("2.7.1"):
-    raise ImportError('[bnlearn] >Error: networkx version should be > 2.7.1.\nTry to: pip install -U networkx')
+if version.parse(nx.__version__) < version.parse("2.7.1"):
+    raise ImportError('[bnlearn] >Error: networkx version should be > 2.7.1\nTry to: pip install -U networkx')
 
+import numpy as np
+if version.parse(np.__version__) < version.parse("1.24.1"):
+    raise ImportError('[bnlearn] >Error: numpy version should be > 1.24.1\nTry to: pip install -U numpy')
+
+# import pandas as pd
+# if version.parse(pd.__version__) > version.parse("1.5.3"):
+#     raise ImportError('[bnlearn] >Error: pands version should be <= 1.5.3')
+
+# This one is moved towards the interactive plot function because it is not required in the setup.
+# import d3blocks as d3
+# if version.parse(d3.__version__) < version.parse("1.4.9"):
+#     raise ImportError('[bnlearn] >Error: d3blocks version should be >= 1.4.9')
 
 # module level doc-string
 __doc__ = """
@@ -113,6 +124,6 @@ References
 * Blog: https://towardsdatascience.com/a-step-by-step-guide-in-detecting-causal-relationships-using-bayesian-structure-learning-in-python-c20c6b31cee5
 * Blog: https://towardsdatascience.com/a-step-by-step-guide-in-designing-knowledge-driven-models-using-bayesian-theorem-7433f6fd64be
 * Github: https://github.com/erdogant/bnlearn
-* Documentation: https://bnlearn.readthedocs.io
+* Documentation: https://erdogant.github.io/bnlearn/
 
 """
