@@ -26,7 +26,7 @@ import warnings
 warnings.filterwarnings("ignore")
 
 
-# %% Sampling from model
+# %% Parameter learning
 def fit(model, df, methodtype='bayes', scoretype='bdeu', smooth=None, n_jobs=-1, verbose=3):
     """Learn the parameters given the DAG and data.
 
@@ -74,6 +74,7 @@ def fit(model, df, methodtype='bayes', scoretype='bdeu', smooth=None, n_jobs=-1,
         Strategy for parameter learning.
             * 'ml', 'maximumlikelihood': Learning CPDs using Maximum Likelihood Estimators.
             * 'bayes': Bayesian Parameter Estimation.
+            * 'DBN': DynamicBayesianNetwork
     scoretype : str, (default : 'bic')
         Scoring function for the search spaces.
             * 'bdeu'
@@ -111,13 +112,22 @@ def fit(model, df, methodtype='bayes', scoretype='bdeu', smooth=None, n_jobs=-1,
     config['method'] = methodtype
     config['n_jobs'] = n_jobs
     adjmat = model['adjmat']
+    independence_test = model.get('independence_test', None)
+
+    # Automatically set methodtype to DBN
+    if str(model['config'].get('method')).lower()=='dbn':
+        config['method'] = 'DBN'
+        if verbose>=3: print('[bnlearn] >Methodtype is set to DynamicBayesianNetwork (DBN)')
 
     if (scoretype=='dirichlet') and (smooth is None):
         raise Exception('[bnlearn] >dirichlet requires "smooth" to be not None')
 
     # Check whether all labels in the adjacency matrix are included from the dataframe
     # adjmat, model = _check_adjmat(model, df)
-    df = bnlearn._filter_df(adjmat, df, verbose=config['verbose'])
+    if config['method']=='DBN':
+        df = adjmat
+    else:
+        df = bnlearn._filter_df(adjmat, df, verbose=config['verbose'])
 
     if config['verbose']>=3: print('[bnlearn] >Parameter learning> Computing parameters using [%s]' %(config['method']))
     # Extract model
@@ -143,6 +153,12 @@ def fit(model, df, methodtype='bayes', scoretype='bdeu', smooth=None, n_jobs=-1,
         for cpd in model.get_cpds():
             if config['verbose']>=3: print("[bnlearn] >CPD of {variable}:".format(variable=cpd.variable))
             if config['verbose']>=3: print(cpd)
+    elif config['method']=='DBN':
+        #  Learning CPDs using Bayesian Parameter Estimation
+        model.fit(df, estimator='MLE')
+        for cpd in model.get_cpds():
+            if config['verbose']>=3: print("[bnlearn] >CPD of {variable}:".format(variable=cpd.variable))
+            if config['verbose']>=3: print(cpd)
     else:
         if config['verbose']>=2: print("[bnlearn] >Warning: methodtype [%s] is unknown. Returning None." %(config['method']))
         return None
@@ -152,6 +168,6 @@ def fit(model, df, methodtype='bayes', scoretype='bdeu', smooth=None, n_jobs=-1,
     out['adjmat'] = adjmat
     out['config'] = config
     out['model_edges'] = list(model.edges())
-    out['structure_scores'] = bnlearn.structure_scores(out, df)
-
-    return(out)
+    out['structure_scores'] = bnlearn.structure_scores(out, df, verbose=verbose)
+    out['independence_test'] = independence_test
+    return out
